/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include "BLEHandler.h"

#include "debug.h"


CYBLE_CONN_HANDLE_T cyBle_connHandle; /* BLE connection handle */

static CYBLE_GAP_BD_ADDR_T privateAddress; /* private address used to advertise */

static CYBLE_GAP_BONDED_DEV_ADDR_LIST_T bondedDeviceList; /* bonded device list */

static CYBLE_GAP_RESOLVING_DEVICE_INFO_T rpaInfo;

static CYBLE_GAP_AUTH_INFO_T peerAuthInfo;

static  uint8 connectionIrk[CYBLE_GAP_SMP_IRK_SIZE]; /* IRK information of a bonded device */

static CYBLE_CUSTOM_CONFIG_T customConfiguration; /* custom configuration for advertising */

static  ADVERTISEMENT_STATE advState; /* state of the advertisment */

static  AUTHENTICATION_STATE authState; /* state use to know if there is a bonded device */

static uint32 u32Passkey = (123456u); /* Fixed password for Bluetooth connection */

static uint16 connInterval = 0u; /* connection interval */

static uint8 u8ConnectionFlag = 0; /* connection is active */

static uint8 u8DeleteBondingFlag = 1; /* flag to signal that the bonding information can be deleted */

static uint8 centralAddressResolutionValue;


/************************ private functions  declaration ************************************/
CY_ISR_PROTO(BLEHandler__vUnpairISRCallback);
void DisplayBondList(void);
static void AdvertisePrivately(CYBLE_GAP_BD_ADDR_T *peerBdAddr);
/************************ private functions definition **************************************/
static void AdvertisePublicly(void)
{
    /* Set the original public address as the device address */
    CyBle_SetDeviceAddress(&cyBle_deviceAddress);
    
    /* Revert if any changes made to advertisement settings */
    cyBle_discoveryModeInfo.advData = &cyBle_discoveryData;
    cyBle_discoveryModeInfo.advParam->advType = CYBLE_GAPP_CONNECTABLE_UNDIRECTED_ADV;
    cyBle_discoveryParam.ownAddrType = CYBLE_GAP_ADDR_TYPE_PUBLIC;
    CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
    
    DBG_PRINT_TEXT("Advertising with public address : ");
    
    for(uint8 counter = CYBLE_GAP_BD_ADDR_SIZE; counter > 0; counter--)
    {
        DBG_PRINT_HEX_TEXT(cyBle_deviceAddress.bdAddr[counter - 1], " ");
    }    
    DBG_PRINT_TEXT("\r\n");
    
#if(CYBLE_LL_PRIVACY_FEATURE_ENABLED)
    /* Disable LL Privacy */
    CyBle_GapSetAddressResolutionEnable(0u);
#endif  /* CYBLE_LL_PRIVACY_FEATURE_ENABLED */
}


static void AdvertisePrivately(CYBLE_GAP_BD_ADDR_T *peerBdAddr)
{
    CYBLE_API_RESULT_T apiResult;
    /* Create a new data structure for Advertisement data 
     * without the device name.
     */
    CYBLE_GAPP_DISC_DATA_T privateAdvData = 
    {
        {           /* advertising_data */
            0x02u,  /* Length of this data */
            0x01u,  /* <<Flags>> */
            0x06u   /* LE General Discoverable Mode, BR/EDR Not Supported */
        },
        3u           /* adv_data_length */
    };
 
    /* Set custom advertising packet */
    cyBle_discoveryModeInfo.advData = &privateAdvData;
    
#if(CYBLE_LL_PRIVACY_FEATURE_ENABLED)
    /* Enable LL Privacy */
    CyBle_GapSetAddressResolutionEnable(1u);

    if((centralAddressResolutionValue != 0u) && ((customConfiguration.mode & CYBLE_CUSTOM_MODE_DIRECTED_ADV) != 0u))
    {   /* Set directed advertising mode */
        cyBle_discoveryModeInfo.advParam->advType = CYBLE_GAPP_CONNECTABLE_LOW_DC_DIRECTED_ADV;
    }
    else
    {   /* Set undirected advertising mode */
        cyBle_discoveryModeInfo.advParam->advType = CYBLE_GAPP_CONNECTABLE_UNDIRECTED_ADV;
    }
    /* Set peer address for direct advertising */
    if((peerBdAddr->type == CYBLE_GAP_ADDR_TYPE_PUBLIC) ||
       (peerBdAddr->type == CYBLE_GAP_ADDR_TYPE_RANDOM))
    {
        memcpy(cyBle_discoveryModeInfo.advParam->directAddr, peerBdAddr->bdAddr, CYBLE_GAP_BD_ADDR_SIZE);
        cyBle_discoveryModeInfo.advParam->directAddrType = peerBdAddr->type;
    }    
#else
    /* Generate a new resolvable private address */
    apiResult = CyBle_GapGenerateDeviceAddress(&privateAddress, CYBLE_GAP_RANDOM_PRIV_RESOLVABLE_ADDR, rpaInfo.localIrk);
    if(apiResult != CYBLE_ERROR_OK)
    {
        DBG_PRINT_HEX_TEXT(apiResult, " CyBle_GapGenerateDeviceAddress API Error\r\n");
    }
    /* Set a new device address */
    apiResult = CyBle_SetDeviceAddress(&privateAddress);
    if(apiResult != CYBLE_ERROR_OK)
    {
        DBG_PRINT_HEX_TEXT(apiResult, " CyBle_SetDeviceAddress API Error\r\n");
    }
    /* peerBdAddr is not used for legacy pairing */
    (void)peerBdAddr;
#endif /* !CYBLE_LL_PRIVACY_FEATURE_ENABLED */
        
    #if(CYBLE_LL_PRIVACY_FEATURE_ENABLED)
        /* Controller generates Resolvable Private Address based on the local IRK from resolving list.*/
        cyBle_discoveryParam.ownAddrType = CYBLE_GAP_ADDR_TYPE_PUBLIC_RPA;
    #else 
        cyBle_discoveryParam.ownAddrType = CYBLE_GAP_ADDR_TYPE_RANDOM;
    #endif /* CYBLE_LL_PRIVACY_FEATURE_ENABLED */
    
    apiResult = CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);

    if(apiResult != CYBLE_ERROR_OK)
    {
        DBG_PRINT_HEX_TEXT(apiResult," CyBle_GapReadLocalResolvableAddress API Error \r\n");
    }
    /* Get a new resolvable device address */
    #if(CYBLE_LL_PRIVACY_FEATURE_ENABLED)
        apiResult = CyBle_GapReadLocalResolvableAddress(peerBdAddr, privateAddress.bdAddr);            
        if(apiResult != CYBLE_ERROR_OK)
        {
            DBG_PRINT_HEX_TEXT(apiResult," CyBle_GapReadLocalResolvableAddress API Error \r\n");
        }
        
        if((centralAddressResolutionValue != 0u) && ((customConfiguration.mode & CYBLE_CUSTOM_MODE_DIRECTED_ADV) != 0u))
        {
            DBG_PRINT_TEXT("Directed Advertising to: ");
            
            for(uint8 counter = CYBLE_GAP_BD_ADDR_SIZE; counter > 0u; counter--)
            {
                DBG_PRINT_HEX_TEXT(peerBdAddr->bdAddr[counter - 1u], " ");
            }
            
            DBG_PRINT_TEXT("\n");
        }
    #else 
        apiResult = CyBle_GetDeviceAddress(&privateAddress);
        if(apiResult != CYBLE_ERROR_OK)
        {
            DBG_PRINT_HEX_TEXT(apiResult, " CyBle_GetDeviceAddress API Error\r\n");
        }
    #endif /* CYBLE_LL_PRIVACY_FEATURE_ENABLED */
    
    DBG_PRINT_TEXT("Advertising with new private address : ");
    
    for(uint8 counter = CYBLE_GAP_BD_ADDR_SIZE; counter > 0; counter--)
    {
        DBG_PRINT_HEX_TEXT(privateAddress.bdAddr[counter - 1], " ");
    }    
    DBG_PRINT_TEXT("\r\n");
}


void DisplayBondList(void)
{
    
    uint8 deviceCount;
    uint8 bondedDeviceBdHandle;
    CYBLE_GAP_BD_ADDR_T bondedDeviceAddress;
    uint8 bondedDeviceKeysFlag;
    CYBLE_API_RESULT_T apiResult;
    
    /* Find out whether the device has bonded information stored already or not */
    apiResult = CyBle_GapGetBondedDevicesList(&bondedDeviceList);
    if(apiResult != CYBLE_ERROR_OK)
    {
        DBG_PRINT_HEX_TEXT(apiResult, " CyBle_GapGetBondedDevicesList API Error: %d \r\n");
    }
    
    deviceCount = bondedDeviceList.count;

    if(deviceCount != 0u)
    {
        uint8 localKeyFlags;
        CYBLE_GAP_SMP_KEY_DIST_T localKeys;
        CYBLE_GAP_SMP_KEY_DIST_T securityKeys;
        
        
        DBG_PRINT_TEXT("Bond List: ");
        
        do
        {
            DBG_PRINT_HEX_TEXT(deviceCount, " \n");
            deviceCount--;
            
            if(bondedDeviceList.bdAddrList[deviceCount].type == CYBLE_GAP_ADDR_TYPE_RANDOM)
            {
                DBG_PRINT_TEXT("Peer Random Address:");
            }
            else
            {
                DBG_PRINT_TEXT("Peer Public Address:");
            }
    
            for(uint8 counter = CYBLE_GAP_BD_ADDR_SIZE; counter > 0; counter--)
            {
                DBG_PRINT_HEX_TEXT(bondedDeviceList.bdAddrList[deviceCount].bdAddr[counter - 1], " ");
            }    
            DBG_PRINT_TEXT("\r\n");
            
            /* Use first in first out (FIFO) method to get the bonded device addresses from the
            * list. The last address retrieved from the bonding list should be the most 
            * recently connected device.
            */
            memcpy(bondedDeviceAddress.bdAddr, bondedDeviceList.bdAddrList[deviceCount].bdAddr, CYBLE_GAP_BD_ADDR_SIZE);
            bondedDeviceAddress.type = bondedDeviceList.bdAddrList[deviceCount].type;

            /* Get the BD handle of currently processed device */
            CyBle_GapGetPeerBdHandle(&bondedDeviceBdHandle, &bondedDeviceAddress);

            /* Get the IRK for currently processed device */
            bondedDeviceKeysFlag |= CYBLE_GAP_KEY_FLAG_LII;   /* Identity Resolution Key */ 
            CyBle_GapGetPeerDevSecurityKeyInfo(bondedDeviceBdHandle, &bondedDeviceKeysFlag, &securityKeys);
            /* peerIrk is used by CyBle_GapGenerateDeviceAddress() */
            memcpy(connectionIrk, securityKeys.irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
            memcpy(rpaInfo.peerIrk, connectionIrk, CYBLE_GAP_SMP_IRK_SIZE);
            
            DBG_PRINT_TEXT("peer IRK:");
            for(uint8 counter = 0u; counter < CYBLE_GAP_SMP_IRK_SIZE; counter++)
            {
                DBG_PRINT_HEX_TEXT(connectionIrk[counter], " ");
            }

            
            /* Get the local keys */
            CyBle_GapGetDevSecurityKeyInfo(&localKeyFlags, &localKeys);

            if((localKeyFlags & CYBLE_GAP_KEY_FLAG_LII) != 0u)
            {
                memcpy(rpaInfo.localIrk, localKeys.irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
                DBG_PRINT_TEXT(", local IRK:");
                for(uint8 counter = 0u; counter < CYBLE_GAP_SMP_IRK_SIZE; counter++)
                {
                    DBG_PRINT_HEX_TEXT(rpaInfo.localIrk[counter], " ");
                }
            }
            
            DBG_PRINT_TEXT("\r\n");
            
        #if(CYBLE_LL_PRIVACY_FEATURE_ENABLED)
            
            memcpy(rpaInfo.bdAddr, bondedDeviceAddress.bdAddr, CYBLE_GAP_BD_ADDR_SIZE);
            rpaInfo.type = bondedDeviceAddress.type;

            /* Add device to resolving list */
            CyBle_GapSetAddressResolutionEnable(0u);
            CyBle_GapAddDeviceToResolvingList(&rpaInfo);
            CyBle_GapSetAddressResolutionEnable(1u);
            
            /* Generate new private address automatically */
            CyBle_GapSetResolvablePvtAddressTimeOut(CYBLE_PVT_ADDRESS_TIMEOUT);
            
        #endif /* CYBLE_LL_PRIVACY_FEATURE_ENABLED */

        } while(deviceCount != 0u);
    }
}


inline static void BLE__vClockConfigInDisconnectState(void)
{
    CYBLE_BLESS_CLK_CFG_PARAMS_T clockConfig;
    CYBLE_API_RESULT_T opResult = CYBLE_ERROR_OK;
    
    /* get the configrated parameter */    
    opResult = CyBle_GetBleClockCfgParam(&clockConfig); 
    /* check api call result */
    if(opResult != CYBLE_ERROR_OK)
    {
        DBG_PRINT_HEX_TEXT(opResult, " Error code API get ble clock cfg\n");
    }
    /*set the device sleep clock acuracy based on the tuned ppm of the WCO */
    clockConfig.bleLlSca = CYBLE_LL_SCA_000_TO_020_PPM;
    /*set clock parameters*/
    opResult = CyBle_SetBleClockCfgParam(&clockConfig);
    if(opResult != CYBLE_ERROR_OK)
    {
        DBG_PRINT_HEX_TEXT(opResult, " Error code API set clock cfg\n");
    }
    /* no connection */
    u8ConnectionFlag = 0; 
    /* Turn off the LED */
    LED_Write(1); 
}

/* Callbacks handler for BLE */
static void BLEStackCallback(uint32 event, void* eventParam)
{
    CYBLE_GAP_SMP_KEY_DIST_T * keys;
    CYBLE_API_RESULT_T opResult = CYBLE_ERROR_OK;
    
    switch(event)
    {
        /* common events */
         case CYBLE_EVT_STACK_ON:
        
            DBG_PRINT_TEXT("CYBLE_EVT_STACK_ON \n");    
        
            BLE__vClockConfigInDisconnectState();
            /*Set the passkey to "123456"*/
            (void)CyBle_GapFixAuthPassKey(FIX_PASSKEY, u32Passkey);
            /* start advertising */
            if(advState == ADVERTISEMENT_PUBLIC)
            {
                AdvertisePublicly();
            }
            else    /* ADVERTISEMENT_PRIVATE */
            {
                /* Display Bond list and initialize Resolving list */
                DisplayBondList();
                AdvertisePrivately(&bondedDeviceList.bdAddrList[0u]);
            }
            break;
            
        case CYBLE_EVT_TIMEOUT:
            DBG_PRINT_TEXT("Timeout: ");
            DBG_PRINT_HEX_TEXT(*(CYBLE_TO_REASON_CODE_T *) eventParam,"\n");
            break;
            
        case CYBLE_EVT_HARDWARE_ERROR:    /* This event indicates that some internal HW error has occurred. */
            DBG_PRINT_TEXT("CYBLE_EVT_HARDWARE_ERROR \r\n");
            break;
            
        case CYBLE_EVT_HCI_STATUS:
            DBG_PRINT_HEX_TEXT(*(uint8 *)eventParam, "EVT_HCI_STATUS\r\n" );
            break;
            
        case CYBLE_EVT_STACK_BUSY_STATUS:
            break;
            
        /**********************************************************
        *                       GAP Events
        ***********************************************************/      
        case CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
            
            DBG_PRINT_TEXT("CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP\n");
            
            /* If advertisement timed out and we were not connected 
             * yet, restart advertisement.
             */
            if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
            {
                if(CyBle_GetState() == CYBLE_STATE_DISCONNECTED)
                {
                    if(advState == ADVERTISEMENT_PUBLIC)
                    {
                        AdvertisePublicly();
                    }
                    else    /* ADVERTISEMENT_PRIVATE */
                    {
                        /* Display Bond list and initialize Resolving list */
                        DisplayBondList();
                        AdvertisePrivately(&bondedDeviceList.bdAddrList[0u]);
                    }
                }
            }
            else
            {
                DBG_PRINT_TEXT("Advertisement stopped. \n");
                authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
            }
            break;    
            
    #if(CYBLE_LL_PRIVACY_FEATURE_ENABLED)          
        case CYBLE_EVT_GAP_ENHANCE_CONN_COMPLETE:
        {
            DBG_PRINT_TEXT("CYBLE_EVT_GAP_ENHANCE_CONN_COMPLETE\n");
            CYBLE_GAP_ENHANCE_CONN_COMPLETE_T *enhConnParameters;
            enhConnParameters = (CYBLE_GAP_ENHANCE_CONN_COMPLETE_T *)eventParam;
            connInterval = enhConnParameters->connIntv;
        }
    #else      
        case CYBLE_EVT_GAP_DEVICE_CONNECTED: 
        {
            DBG_PRINT_TEXT("CYBLE_EVT_GAP_DEVICE_CONNECTED\n");
            CYBLE_GAP_CONN_PARAM_UPDATED_IN_CONTROLLER_T *connParameters;
            connParameters = (CYBLE_GAP_CONN_PARAM_UPDATED_IN_CONTROLLER_T *)eventParam;
            connInterval = connParameters->connIntv;
        }
    #endif  /* CYBLE_LL_PRIVACY_FEATURE_ENABLED */
            /* Continue connection case without a break */   
    
            /* Initiate pairing process */
            if((cyBle_authInfo.security & CYBLE_GAP_SEC_LEVEL_MASK) > CYBLE_GAP_SEC_LEVEL_1)
            {
                opResult = CyBle_GapAuthReq(cyBle_connHandle.bdHandle, &cyBle_authInfo);
                if(opResult != CYBLE_ERROR_OK)
                {
                    DBG_PRINT_HEX_TEXT(opResult, " CyBle_GapAuthReq API Error \r\n");
                }
            }
            else
            {
                authState = AUTHENTICATION_UPDATE_CONN_PARAM;
            }
            break;
        
        case CYBLE_EVT_GAPC_CONNECTION_UPDATE_COMPLETE:       
            DBG_PRINT_TEXT("CYBLE_EVT_GAPC_CONNECTION_UPDATE_COMPLETE\n");
            break;
            
        case CYBLE_EVT_L2CAP_CONN_PARAM_UPDATE_RSP:
            DBG_PRINT_HEX_TEXT(*(uint16 *)eventParam == 0u ? 1 : 0, " CYBLE_EVT_L2CAP_CONN_PARAM_UPDATE_RSP");
            break;    
            
        case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:
            DBG_PRINT_HEX_TEXT(*(uint8 *)eventParam, " CYBLE_EVT_GAP_DEVICE_DISCONNECTED\n");  
            
            /* Disconnected but bonding is not complete; restart 
             * advertisement.
             */
            if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
            {
                if(advState == ADVERTISEMENT_PUBLIC)
                {
                    AdvertisePublicly();
                }
                else    /* ADVERTISEMENT_PRIVATE */
                {
                    /* Display Bond list and initialize Resolving list */
                    DisplayBondList();
                    AdvertisePrivately(&bondedDeviceList.bdAddrList[0u]);
                }
            }
            else
            {
                authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
            }
            
            LED_Write(1); /* Turn off the LED */
            break;
             
        case CYBLE_EVT_GAP_AUTH_REQ: 
            DBG_PRINT_TEXT("CYBLE_EVT_GAP_AUTH_REQ from Master:\r\n");
            peerAuthInfo = *(CYBLE_GAP_AUTH_INFO_T *)eventParam;
            break;

    #if(CYBLE_SECURE_CONN_FEATURE_ENABLED)  
         case CYBLE_EVT_GAP_SMP_NEGOTIATED_AUTH_INFO:
            DBG_PRINT_TEXT("SMP Negotiated \n");
            CYBLE_GAP_AUTH_INFO_T *info = (CYBLE_GAP_AUTH_INFO_T *)eventParam;
            DBG_PRINT_HEX_TEXT(info->authErr, "  SMT Auth error\n");
            DBG_PRINT_HEX_TEXT( info->security, "  SMT SECURITY \n");
            DBG_PRINT_HEX_TEXT( info->bonding, "  SMT BONDING \n");
            DBG_PRINT_HEX_TEXT( info->pairingProperties, "  SMT PAIRING PROP \n");
            DBG_PRINT_HEX_TEXT( info->ekeySize, "  SMT ekey size \n");
            break;
    #endif /* CYBLE_SECURE_CONN_FEATURE_ENABLED */        
                
        case CYBLE_EVT_GAP_PASSKEY_DISPLAY_REQUEST:
                DBG_PRINT_TEXT("passkey display request\n");  
            break;       

         case CYBLE_EVT_GAP_KEYINFO_EXCHNGE_CMPLT:
            DBG_PRINT_TEXT("CYBLE_EVT_GAP_KEYINFO_EXCHNGE_CMPLT \r\n");
            /* Copy the IRK key to a buffer; to be used for creating a private address.*/
            keys = (CYBLE_GAP_SMP_KEY_DIST_T *) eventParam;
            memcpy(connectionIrk, keys->irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
        break;
  
        case CYBLE_EVT_GAP_AUTH_COMPLETE:
            DBG_PRINT_TEXT("Authentication complete.\n");
            /* Authentication complete; initiate bonding */
            peerAuthInfo = *(CYBLE_GAP_AUTH_INFO_T *)eventParam;
            /* Check that peer device supports bonding */
            if(peerAuthInfo.bonding != 0u)
            {
                authState = AUTHENTICATION_COMPLETE_BONDING_REQD;
                #if(CYBLE_LL_PRIVACY_FEATURE_ENABLED)
                {
                    uint8 localKeyFlags;
                    CYBLE_GAP_SMP_KEY_DIST_T localKeys;
                    CYBLE_GAP_BD_ADDR_T peerBdAddr;
                    
                    memcpy(rpaInfo.peerIrk, connectionIrk, CYBLE_GAP_SMP_IRK_SIZE);
                    
                    CyBle_GapGetDevSecurityKeyInfo(&localKeyFlags, &localKeys);
                    if((localKeyFlags & 1u) != 0u)
                    {
                        memcpy(rpaInfo.localIrk, localKeys.irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
                    }
                    CyBle_GapGetPeerBdAddr(cyBle_connHandle.bdHandle, &peerBdAddr);
                    memcpy(rpaInfo.bdAddr, peerBdAddr.bdAddr, CYBLE_GAP_BD_ADDR_SIZE);
                    rpaInfo.type = peerBdAddr.type;
                    
                    /* Add device to resolving list */
                    CyBle_GapAddDeviceToResolvingList(&rpaInfo);
                    /* Generate new private address automatically */
                    CyBle_GapSetResolvablePvtAddressTimeOut(CYBLE_PVT_ADDRESS_TIMEOUT); 
                }
                #endif /* CYBLE_LL_PRIVACY_FEATURE_ENABLED */
            }
            else /* Use public advertisement if bonding is not supported by peer device */
            {
                advState = ADVERTISEMENT_PUBLIC;
                authState = AUTHENTICATION_UPDATE_CONN_PARAM;                
            }
            break;
        
        case CYBLE_EVT_GAP_AUTH_FAILED:
            DBG_PRINT_TEXT("Authentication failed.\n");
            opResult = CyBle_GapDisconnect(cyBle_connHandle.bdHandle);
            if(opResult != CYBLE_ERROR_OK)
            {
                DBG_PRINT_HEX_TEXT(opResult, " Disconnect code\n");
            }
            /* Update authentication state so that bond information could be removed later */
            authState = AUTHENTICATION_BONDING_COMPLETE;
            /*Authentication Failed.Display the error code */
            DBG_PRINT_HEX_TEXT(*(uint8 *)eventParam,"\n");
            break;    
                     
            
        case CYBLE_EVT_GAP_ENCRYPT_CHANGE:
                /*Encyrption Change event*/
                DBG_PRINT_TEXT("\n");
                /*Get the current status of Encryption*/
                if(*(uint8 *)eventParam==0x00)
                {
                    DBG_PRINT_TEXT("Encrytpion OFF\n");
                }
                else if(*(uint8 *)eventParam==0x01)
                {
                    DBG_PRINT_TEXT("Encryption ON\n");
                }
                else
                {
                    DBG_PRINT_TEXT("ERROR\n");
                }
            break;    
                
    #if(CYBLE_DLE_FEATURE_ENABLED)
        case CYBLE_EVT_GAP_DATA_LENGTH_CHANGE:
            DBG_PRINT_TEXT("DATA_LENGHT_CHANGE\n");
            break;
            
        case CYBLE_EVT_GAP_KEYPRESS_NOTIFICATION:
            DBG_PRINT_TEXT("CYBLE_EVT_GAP_KEYPRESS_NOTIFICATION\n");
            break;
    #endif /* CYBLE_DLE_FEATURE_ENABLED */
        /**********************************************************
        *                       GATT Events
        ***********************************************************/
        case CYBLE_EVT_GATT_CONNECT_IND:
            DBG_PRINT_TEXT("connection indication\n");
            u8ConnectionFlag = 1;
            cyBle_connHandle = *(CYBLE_CONN_HANDLE_T *) eventParam;
            LED_Write(0); /* Turn on the LED */
            break;
            
        case CYBLE_EVT_GATT_DISCONNECT_IND:
            DBG_PRINT_TEXT("CYBLE_EVT_GATT_DISCONNECT_IND \r\n");
            break;
            
        case CYBLE_EVT_GATTS_XCNHG_MTU_REQ:
                DBG_PRINT_TEXT("CYBLE_EVT_GATTS_XCNHG_MTU_REQ\r\n");
            break;
                
        case CYBLE_EVT_GATTC_READ_BY_TYPE_RSP:
            {
                CYBLE_GATTC_GRP_ATTR_DATA_LIST_T *locAttrData;
                
                locAttrData = &(*(CYBLE_GATTC_READ_BY_GRP_RSP_PARAM_T *)eventParam).attrData;
                
                DBG_PRINT_HEX_TEXT(locAttrData->attrLen," EVT_GATT_READ_BY_TYPE_RSP\n");
                for(uint8 counter = 0u; counter < locAttrData->attrLen; counter++)
                { 
                    DBG_PRINT_HEX_TEXT(*(uint8 *)(locAttrData->attrValue + counter), " ");
                }
                DBG_PRINT_TEXT("\r\n");

                if(authState == AUTHENTICATION_READING_CAR)
                {
                    if(locAttrData->attrLen > CYBLE_DB_ATTR_HANDLE_LEN)
                    {
                        centralAddressResolutionValue = *(uint8 *)(locAttrData->attrValue + CYBLE_DB_ATTR_HANDLE_LEN);
                        /* Central Address Resolution characteristic is supported by peer device */
                         DBG_PRINT_HEX_TEXT(centralAddressResolutionValue," Central Address Resolution value\r\n");
                    }
                    authState = AUTHENTICATION_UPDATE_CONN_PARAM;
                }
            }
            break;        
                
        case CYBLE_EVT_GATTC_ERROR_RSP:
            if(authState == AUTHENTICATION_READING_CAR)
            {
                /* Central Address Resolution is not supported by peer device */
                DBG_PRINT_TEXT("Central Address Resolution characteristic is absent \r\n");
                authState = AUTHENTICATION_UPDATE_CONN_PARAM;
            }
            break;        
    
        case CYBLE_EVT_GATTS_INDICATION_ENABLED:
            DBG_PRINT_TEXT("CYBLE_EVT_GATTS_INDICATION_ENABLED \r\n");
            break;
            
        case CYBLE_EVT_GATTS_WRITE_REQ:      
            DBG_PRINT_TEXT("CYBLE_EVT_GATTS_WRITE\r\n"); 
            break;
            
        case CYBLE_EVT_GATTS_HANDLE_VALUE_CNF:
            DBG_PRINT_TEXT("CYBLE_EVT_GATTS_HANDLE_VALUE_CNF\r\n"); 
            break;
            
        case CYBLE_EVT_GATTS_READ_CHAR_VAL_ACCESS_REQ:
            DBG_PRINT_TEXT("CYBLE_EVT_GATTS_READ_CHAR_VAL_ACCESS_REQ\r\n");
            break;    
            
        default:
            /* should not be here */
        break;
    }
}

/************************ Public functions definition **************************************/
void BLEHandler_vStart(void)
{
    CYBLE_API_RESULT_T opResult = CYBLE_ERROR_OK;
    
    /* Start the BLE with the call back associated */
    opResult = CyBle_Start(BLEStackCallback);
     /* check error code */
    if(opResult != CYBLE_ERROR_OK) 
    {
        DBG_PRINT_DEC_TEXT(opResult, " code Error start BLE\n");
    }
    
#if(CYBLE_BONDING_REQUIREMENT == CYBLE_BONDING_YES)
    /* Find out whether the device has bonded information stored already or not */
    opResult = CyBle_GapGetBondedDevicesList(&bondedDeviceList);
    /* check error code */
    if(opResult != CYBLE_ERROR_OK) 
    {
        DBG_PRINT_DEC_TEXT(opResult, " code Error get bonded list\n");
    }
    
    if(bondedDeviceList.count != 0)
    {
        /* Generate new private address automatically */
        CyBle_GapSetResolvablePvtAddressTimeOut(CYBLE_PVT_ADDRESS_TIMEOUT);
        /* Start the isr on pin in order to catch unpair event */
        Unpair_ISR_StartEx(BLEHandler__vUnpairISRCallback);
        /* Start advertisement with private address */
        advState = ADVERTISEMENT_PRIVATE;
        authState = AUTHENTICATION_BONDING_COMPLETE;
    }
    else
    {
        /* Start advertisement with public address */
        advState = ADVERTISEMENT_PUBLIC;
        authState = AUTHENTICATION_NOT_CONNECTED;
    }
#endif /* CYBLE_BONDING_REQUIREMENT == CYBLE_BONDING_YES */

}

/* this should be called every time something was done with the BLE */
void BLEHandler_vPrivacyTask(void)
{
    /* All zeros passed as  argument passed to CyBle_GapRemoveDeviceFromWhiteList for removing all the bonding data stored */
    CYBLE_GAP_BD_ADDR_T clearAllDevices = {{0,0,0,0,0,0},0};
    CYBLE_API_RESULT_T opResult = CYBLE_ERROR_OK;
    
    
#if(CYBLE_BONDING_REQUIREMENT == CYBLE_BONDING_YES)
    if(cyBle_pendingFlashWrite != 0u) 
    {
        /* Store bonding data of the current connection */
        while(CYBLE_ERROR_OK != CyBle_StoreBondingData(0u));  
    }
#endif /* CYBLE_BONDING_REQUIREMENT == CYBLE_BONDING_YES */
    
    
    switch(authState)
    {  
        case AUTHENTICATION_COMPLETE_BONDING_REQD:    
        
            authState = AUTHENTICATION_READ_CENTRAL_ADDRESS_RESOLUTION;
            DBG_PRINT_TEXT("Bonding complete. Press SW push at any time to remove this bond. \r\n");
            
            /* Ready for private advertisement the next time */
            advState = ADVERTISEMENT_PRIVATE;
            
            Unpair_ISR_StartEx(BLEHandler__vUnpairISRCallback);
            break;
            
            
        case AUTHENTICATION_BONDING_COMPLETE:
            if(!u8DeleteBondingFlag)
            {
                /* User wants the bond to be removed */
                DBG_PRINT_TEXT("Remove the bond. \r\n");
                if(CyBle_GetState() == CYBLE_STATE_CONNECTED)
                {
                    /* Disconnect */
                    authState = AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT;
                    CyBle_GapDisconnect(cyBle_connHandle.bdHandle);
                }
                else if(CyBle_GetState() == CYBLE_STATE_ADVERTISING)
                {
                    /* Stop advertisement */
                    authState = AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT;
                    CyBle_GappStopAdvertisement();
                }
                else
                {
                    authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                }
                
                u8DeleteBondingFlag = 1;
            }
            break;

            
        case AUTHENTICATION_BONDING_REMOVE_GO_AHEAD:
        #if(CYBLE_BONDING_REQUIREMENT == CYBLE_BONDING_YES)
            /* Remove all bonded devices in the list */
            CyBle_GapRemoveBondedDevice(&clearAllDevices);
            
            Unpair_ISR_Stop();

            #if(CYBLE_LL_PRIVACY_FEATURE_ENABLED)
                /* Remove all bonded devices in the resolving list */
                CyBle_GapClearResolvingList();
            #endif  /* CYBLE_LL_PRIVACY_FEATURE_ENABLED */

            /* Process pending BLE events */
            CyBle_ProcessEvents();

            DBG_PRINT_TEXT("Clear the list of bonded devices. \r\n");
            
            /* Clear the IRK used for random private address creation */
            memset(connectionIrk, 0, CYBLE_GAP_SMP_IRK_SIZE);
        #endif  /* CYBLE_BONDING_REQUIREMENT == CYBLE_BONDING_YES */

            /* Start advertising with a public address */
            advState = ADVERTISEMENT_PUBLIC;
            AdvertisePublicly();
                
            authState = AUTHENTICATION_NOT_CONNECTED;
            break;
            
        case AUTHENTICATION_READ_CENTRAL_ADDRESS_RESOLUTION:
        {
            CYBLE_GATTC_READ_BY_TYPE_REQ_T readByTypeReqParam;
            
            centralAddressResolutionValue = 0u;
            readByTypeReqParam.range.startHandle = CYBLE_GATT_ATTR_HANDLE_START_RANGE;
            readByTypeReqParam.range.endHandle = CYBLE_GATT_ATTR_HANDLE_END_RANGE;
            readByTypeReqParam.uuid.uuid16 = CYBLE_UUID_CHAR_CENTRAL_ADDRESS_RESOLUTION;
            readByTypeReqParam.uuidFormat = CYBLE_GATT_16_BIT_UUID_FORMAT;
            opResult = CyBle_GattcReadUsingCharacteristicUuid(cyBle_connHandle, &readByTypeReqParam);
            if(opResult == CYBLE_ERROR_OK)
            {
                authState = AUTHENTICATION_READING_CAR;
            }
            else
            {
                DBG_PRINT_HEX_TEXT(opResult, " CyBle_GattcReadUsingCharacteristicUuid API Error r\n");
                authState = AUTHENTICATION_UPDATE_CONN_PARAM;
            }
        }
            break;
        
        case AUTHENTICATION_UPDATE_CONN_PARAM:
            if(connInterval > CYBLE_GAPP_CONNECTION_INTERVAL_MIN)
            {
                CYBLE_GAP_CONN_UPDATE_PARAM_T connUpdateParam;
                /* Send Connection Parameter Update Request to Client */
                connUpdateParam.connIntvMin = CYBLE_GAPP_CONNECTION_INTERVAL_MIN;
                connUpdateParam.connIntvMax = CYBLE_GAPP_CONNECTION_INTERVAL_MAX;
                connUpdateParam.connLatency = CYBLE_GAPP_CONNECTION_SLAVE_LATENCY;
                connUpdateParam.supervisionTO = CYBLE_GAPP_CONNECTION_TIME_OUT;
                
                opResult = CyBle_L2capLeConnectionParamUpdateRequest(cyBle_connHandle.bdHandle, &connUpdateParam);
                if(opResult == CYBLE_ERROR_OK)
                {
                    authState = AUTHENTICATION_BONDING_COMPLETE;
                }
                else
                {
                    DBG_PRINT_HEX_TEXT(opResult, " CyBle_L2capLeConnectionParamUpdateRequest API Error \r\n");
                }
            }   
            else
            {
                authState = AUTHENTICATION_BONDING_COMPLETE;
            }
            break;
            
        default:    /* Not supported state */
            break;
    }
}

/* ISR */
CY_ISR(BLEHandler__vUnpairISRCallback)
{
    u8DeleteBondingFlag = 0; /* set the flag in order to clear the bonding data */
   
    UnpairPin_ClearInterrupt(); /* clear interrupt on pin */
    Unpair_ISR_ClearPending(); /* clear pending */
}

/* [] END OF FILE */
